using System.Collections.Generic;

namespace Aurora
{
    public enum SortDirection {
        Asc,
        Desc,
    }

    public class CursorList<T> : List<T>
    {
        private string _orderBy;
        private SortDirection _direction;
        private string _previousPageToken;
        private string _nextPageToken;
        private int _pageSize;

        public CursorList(){
            this._direction = SortDirection.Desc;
            this._orderBy = string.Empty;
            this._previousPageToken = string.Empty;
            this._nextPageToken = string.Empty;
            this._pageSize = 10;
        }

        public CursorList<T> WithNextPage(string nextPageToken){
            this._nextPageToken = nextPageToken;
            return this;
        }

        public CursorList<T> WithPreviousPage(string prevPageToken){
            this._previousPageToken = prevPageToken;
            return this;
        }

        public CursorList<T> WithSort(string orderBy, SortDirection direction ) {
            this._orderBy = orderBy;
            this._direction = direction;
            return this;
        }

        public CursorList<T> WithSize(int size){
            this._pageSize = size;
            return this;
        }

        public List<T> Get(){
            if(this._nextPageToken != string.Empty && this._previousPageToken != string.Empty){
                throw new System.InvalidOperationException("Cannot specify both next and previous page tokens");
            }

            List<T> tmpList = new List<T>(this);


            tmpList.Sort(delegate(T first, T second){
                object firstVal = first.GetType().GetProperty(this._orderBy).GetValue(first, null);
                object secondVal = first.GetType().GetProperty(this._orderBy).GetValue(second, null);
                int compare = 0;

                if(firstVal == null && secondVal == null)
                {
                    compare = 0;
                } 
                else if(firstVal == null)
                {
                    compare = 1;
                } 
                else if(secondVal == null)
                {
                    compare = -1;
                }
                else 
                {
                    
                    // Determine number or string types
                    if(firstVal is string)
                    {
                        string firstStr = firstVal as string;
                        string secondStr = secondVal as string;
                        compare = firstStr.CompareTo(secondStr);
                    } else if(firstVal is int)
                    {
                        int? firstInt = firstVal as int?;
                        int? secondInt = secondVal as int?;
                        compare = firstInt > secondInt ? 1 : -1;
                    }
                }
                return this._direction == SortDirection.Asc ? compare : compare * -1;
            });

            return tmpList.GetRange(0, this._pageSize > tmpList.Count ? tmpList.Count : this._pageSize);
        }
    }
}